@@ -2,6 +2,7 @@ from typing import Dict
 
 from .style import Style
 
+
 DEFAULT_STYLES: Dict[str, Style] = {
     "none": Style.null(),
     "reset": Style(
@@ -82,6 +83,9 @@ DEFAULT_STYLES: Dict[str, Style] = {
     "repr.none": Style(color="magenta", italic=True),
     "repr.url": Style(underline=True, color="bright_blue", italic=False, bold=False),
     "repr.uuid": Style(color="bright_yellow", bold=False),
+    "repr.call": Style(color="magenta", bold=True),
+    "repr.path": Style(color="magenta"),
+    "repr.filename": Style(color="bright_magenta"),
     "rule.line": Style(color="bright_green"),
     "rule.text": Style.null(),
     "prompt": Style.null(),
@@ -94,8 +98,6 @@ DEFAULT_STYLES: Dict[str, Style] = {
     "scope.key": Style(color="yellow", italic=True),
     "scope.key.special": Style(color="yellow", italic=True, dim=True),
     "scope.equals": Style(color="red"),
-    "repr.path": Style(color="magenta"),
-    "repr.filename": Style(color="bright_magenta"),
     "table.header": Style(bold=True),
     "table.footer": Style(bold=True),
     "table.cell": Style.null(),
@@ -125,9 +127,6 @@ DEFAULT_STYLES: Dict[str, Style] = {
     "status.spinner": Style(color="green"),
     "tree": Style(),
     "tree.line": Style(),
-}
-
-MARKDOWN_STYLES = {
     "markdown.paragraph": Style(),
     "markdown.text": Style(),
     "markdown.emph": Style(italic=True),
@@ -151,6 +150,3 @@ MARKDOWN_STYLES = {
     "markdown.link": Style(color="bright_blue"),
     "markdown.link_url": Style(color="blue"),
 }
-
-
-DEFAULT_STYLES.update(MARKDOWN_STYLES)
